@@ -1,12 +1,12 @@
-#!/usr/bin/perl
+#!/usr/bin/perl -w
+
+use constant NUM_TESTS => 74;
 
 use strict;
 use Test::More;
 use DBI;
 
-# use warnings;
 # use Data::Dumper;
-
 # $Class::Tables::SQL_DEBUG++;
 
 ############################
@@ -67,7 +67,7 @@ if (not $dbh) {
 		   . "the connection parameters for the test database.";
 
 } else {
-    plan tests => 67;
+    plan tests => NUM_TESTS;
     diag "Starting test suite. Run `perl Makefile.PL -s` to reconfigure "
        . "connection parameters for the test database.";
 }
@@ -115,7 +115,7 @@ $dbh->do($_) for (split /\s*;\s*/, <<"END_OF_SQL");
     create table products (
         id              $pkey,
         name            varchar(50) not null,
-        weight          integer not null,
+        weight          integer,
         price           decimal
     );
     
@@ -289,6 +289,9 @@ is( Employees->search( name => "Frodo Baggins" )->name,
 ok( scalar Employees->search(department => Departments->fetch(3)),
     "search with object constraint on foreign key" );
 
+ok( scalar Employees->search(purchases => 1, foo => "bar"),
+    "search with bogus/evil constraint name" );
+
 ########################
 ## instance accessors ##
 ########################
@@ -305,7 +308,8 @@ my %simple_accessors = (
         name         => '',
         department   => "Departments",
         photo        => '',
-        purchases    => "Purchases" # x
+        purchases    => "Purchases", # x
+        products     => "Products"   # n-to-n
     },
     Purchases => {
         id          => '',
@@ -320,7 +324,8 @@ my %simple_accessors = (
         name        => '',
         weight      => '',
         price       => '',
-        purchases   => "Purchases" # x
+        purchases   => "Purchases", # x
+        employees   => "Employees"  # n-to-n
     }
 );
 
@@ -348,6 +353,18 @@ for my $class (keys %simple_accessors) {
 ok( $field_ok,
     "field() accessor with no args" );
 
+## n-to-n accessors:
+
+{
+    my @long_way  = map { $_->product } Employees->fetch(1)->purchases;
+    my @short_way = Employees->fetch(1)->products;
+    
+    is_deeply(
+        [ map { $_->id } @long_way ],
+        [ map { $_->id } @short_way ],
+        "n-to-n accessors" );
+}
+
 ####
 
 my $h = Employees->fetch(1);
@@ -522,6 +539,30 @@ is( scalar Employees->search,
     undef,
     "delete all in a table" );
 
+#######################
+## undef/null values ##
+#######################
+
+{
+    my $prod = Products->fetch(4);
+    $prod->weight(undef);
+
+    my $ar = $dbh->selectall_arrayref("select weight from products where id=4");
+    
+    ok( exists $ar->[0][0] && ! defined $ar->[0][0],
+        "undef becomes NULL in database" );
+
+}
+
+## flyweight object goes out of scope, so next line must query the database
+
+ok( ! defined Products->fetch(4)->weight,
+    "NULL database values come back undef" );
+
+is( scalar( () = Products->search( weight => undef ) ),
+    1,
+    "search method handles searching with IS NULL" );
+
 ###########################
 ## subclassing revisited ##
 ###########################
@@ -534,7 +575,7 @@ isnt(
 ## done!
 
 $timer = times - $timer;
-diag "SUMMARY: $Class::Tables::SQL_QUERIES queries, ${timer}s ($driver)";
+diag "SUMMARY: $Class::Tables::SQL_QUERIES queries, ${timer}s (using $driver)";
 
 #############
 ## cleanup ##
